angular.module('terralalune',
               [
                 'ionic',
                 'terralalune.services',
                 'terralalune.controllers.general',
                 'terralalune.controllers.track',
                 'terralalune.controllers.account',
                 'terralalune.directives',
                 'terralalune.filters',
                 'LocalForageModule'
               ])

    .run(function ($ionicPlatform, DeviceService) {
           $ionicPlatform.ready(function () {
             // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
             // for form inputs)
             if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
               cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
               cordova.plugins.Keyboard.disableScroll(true);

             }
             if (window.StatusBar) {
               // org.apache.cordova.statusbar required
               StatusBar.styleLightContent();
             }
             DeviceService.init();

             $ionicPlatform.registerBackButtonAction(function () {
               //navigator.app.exitApp();
             }, 100);

           });
         })

    .constant('AuthRejectedCode', 'auth.rejected')

    .config(function ($stateProvider, $urlRouterProvider, AuthRejectedCode, $ionicConfigProvider) {

    		$ionicConfigProvider.backButton.text('').icon('ion-ios7-arrow-left');
    		$ionicConfigProvider.backButton.previousTitleText(false);
    		
    	
              // -- auth resolving for views which need user auth
              // catch by TerralaluneController if not resolved, redirecting on login page
              var resolveAuth = function ($q, TerralaluneApp) {
                var deferred = $q.defer();
                if (TerralaluneApp.isAuth()) {
                  deferred.resolve();
                }
                else {
                  deferred.reject(AuthRejectedCode);
                }
                return deferred.promise;
              };


              // -- general
              $stateProvider.state('home', {
                url: '/',
                templateUrl: 'templates/general/home.html',
                cache: false
              });
              $stateProvider.state('assistance', {
                url: '/assistance',
                templateUrl: 'templates/general/assistance.html',
                cache: false
              });
              $stateProvider.state('myPerformances', {
                url: '/myperformances',
                templateUrl: 'templates/general/myPerformances.html',
                controller: 'StateMyPerformancesController',
                resolve: {
                  resolveAuth: resolveAuth,
                  performancesData: function (TerralaluneApp, CoveaAPI, $q) {
                    var deferred = $q.defer();
                    CoveaAPI.api.user.get(TerralaluneApp.userId)
                        .then(function (data) {
                                deferred.resolve(data.result);
                              },
                              function (msg) {
                                console.error(msg);
                                deferred.reject(msg);
                              });

                    return deferred.promise;
                  }
                },
                cache: false
              });
              $stateProvider.state('teamPerformances', {
                url: '/teamPerformances',
                templateUrl: 'templates/general/teamPerformances.html',
                controller: 'StateTeamPerformancesController',
                resolve: {
                  resolveAuth: resolveAuth,
                  teamData: function (TerralaluneApp, CoveaAPI, $q) {
                    var deferred = $q.defer();
                    CoveaAPI.api.team.get(TerralaluneApp.teamId)
                        .then(function (data) {
                                deferred.resolve(data.result);
                              },
                              function (msg) {
                                console.error(msg);
                                deferred.reject(msg);
                              });

                    return deferred.promise;
                  },
                  newMembresData: function (TerralaluneApp, CoveaAPI, $q) {
                    var deferred = $q.defer();
                    CoveaAPI.api.team.getNewUsers(TerralaluneApp.teamId)
                        .then(function (data) {
                                deferred.resolve(data.result);
                              },
                              function (msg) {
                                console.error(msg);
                                deferred.reject(msg);
                              });

                    return deferred.promise;
                  },
                  teamRankingData: function (TerralaluneApp, CoveaAPI, $q) {
                    var deferred = $q.defer();
                    CoveaAPI.api.team.getRanking()
                        .then(function (data) {
                                deferred.resolve(data.result);
                              },
                              function (msg) {
                                console.error(msg);
                                deferred.reject(msg);
                              });

                    return deferred.promise;
                  }
                },
                cache: false
              });


              // -- account
              $stateProvider.state('signup', {
                url: '/signup',
                templateUrl: 'templates/account/signup.html',
                cache: false
              });
              $stateProvider.state('login', {
                url: '/login',
                templateUrl: 'templates/account/login.html',
                cache: false
              });
              $stateProvider.state('passwordReset', {
                url: '/passwordReset',
                templateUrl: 'templates/account/passwordReset.html',
                cache: false
              });
              $stateProvider.state('inviteColleague', {
                url: '/inviteColleague',
                templateUrl: 'templates/account/inviteColleague.html',
                controller: 'StateInviteColleagueController',
                resolve: {
                  resolveAuth: resolveAuth,
                  nbInvitationsLeft: function (TerralaluneApp, CoveaAPI, $q) {
                    var deferred = $q.defer();
                    CoveaAPI.api.user.nbInvitationsLeft(TerralaluneApp.userId)
                        .then(function (data) {
                                deferred.resolve(data.result);
                              },
                              function (msg) {
                                console.error(msg);
                                deferred.reject(msg);
                              });

                    return deferred.promise;
                  }
                },
                cache: false
              });
              $stateProvider.state('changePassword', {
                url: '/myAccount',
                templateUrl: 'templates/account/changePassword.html',
                resolve: {resolveAuth: resolveAuth},
                cache: false
              });
              $stateProvider.state('logout', {
                url: '/logout',
                controller: 'StateLogoutController',
                resolve: {resolveAuth: resolveAuth},
                cache: false
              });


              // -- track
              $stateProvider.state('newTrack', {
                url: '/newTrack',
                templateUrl: 'templates/track/newTrack.html',
                resolve: {resolveAuth: resolveAuth},
                cache: false
              });
              $stateProvider.state('newGPSTrack', {
                url: '/newGPSTrack',
                templateUrl: 'templates/track/newGPSTrack.html',
                resolve: {resolveAuth: resolveAuth},
                cache: false
              });
              $stateProvider.state('GPSTrackInprogress', {
                url: '/GPSTrackInprogress/:type',
                templateUrl: 'templates/track/GPSTrackInprogress.html',
                controller: 'StateGPSTrackInprogressController as stateGPSTrackInprogressController',
                controllerAs: '',
                resolve: {resolveAuth: resolveAuth},
                cache: false
              });
              $stateProvider.state('GPSTrackFinish', {
                url: '/GPSTrackFinish/:type',
                templateUrl: 'templates/track/GPSTrackFinish.html',
                controller: 'StateGPSTrackFinishController as stateGPSTrackFinishController',
                resolve: {resolveAuth: resolveAuth},
                cache: false
              });
              $stateProvider.state('newManualTrack', {
                url: '/newManualTrack',
                templateUrl: 'templates/track/newManualTrack.html',
                resolve: {resolveAuth: resolveAuth},
                cache: false
              });


              // if none of the above states are matched, use this as the fallback
              $urlRouterProvider.otherwise('/');
            });
